@@ -113,6 +113,7 @@ enum
     FWKNOP_CLI_ARG_KEY_HMAC_BASE64,
     FWKNOP_CLI_ARG_KEY_HMAC,
     FWKNOP_CLI_ARG_USE_HMAC,
+    FWKNOP_CLI_ARG_USE_WGET_USER_AGENT,
     FWKNOP_CLI_ARG_KEY_FILE,
     FWKNOP_CLI_ARG_HMAC_KEY_FILE,
     FWKNOP_CLI_ARG_NAT_ACCESS,
@@ -123,6 +124,10 @@ enum
     FWKNOP_CLI_ARG_NAT_PORT,
     FWKNOP_CLI_ARG_VERBOSE,
     FWKNOP_CLI_ARG_RESOLVE_IP_HTTP,
+    FWKNOP_CLI_ARG_RESOLVE_IP_HTTPS,
+    FWKNOP_CLI_ARG_RESOLVE_HTTP_ONLY,
+    FWKNOP_CLI_ARG_WGET_CMD,
+    FWKNOP_CLI_ARG_NO_SAVE_ARGS,
     FWKNOP_CLI_LAST_ARG
 } fwknop_cli_arg_t;
 
@@ -156,6 +161,7 @@ static fko_var_t fko_var_array[FWKNOP_CLI_LAST_ARG] =
     { "HMAC_KEY_BASE64",       FWKNOP_CLI_ARG_KEY_HMAC_BASE64       },
     { "HMAC_KEY",              FWKNOP_CLI_ARG_KEY_HMAC              },
     { "USE_HMAC",              FWKNOP_CLI_ARG_USE_HMAC              },
+    { "USE_WGET_USER_AGENT",   FWKNOP_CLI_ARG_USE_WGET_USER_AGENT   },
     { "KEY_FILE",              FWKNOP_CLI_ARG_KEY_FILE              },
     { "HMAC_KEY_FILE",         FWKNOP_CLI_ARG_HMAC_KEY_FILE         },
     { "NAT_ACCESS",            FWKNOP_CLI_ARG_NAT_ACCESS            },
@@ -165,7 +171,11 @@ static fko_var_t fko_var_array[FWKNOP_CLI_LAST_ARG] =
     { "NAT_RAND_PORT",         FWKNOP_CLI_ARG_NAT_RAND_PORT         },
     { "NAT_PORT",              FWKNOP_CLI_ARG_NAT_PORT              },
     { "VERBOSE",               FWKNOP_CLI_ARG_VERBOSE               },
-    { "RESOLVE_IP_HTTP",       FWKNOP_CLI_ARG_RESOLVE_IP_HTTP       }
+    { "RESOLVE_IP_HTTP",       FWKNOP_CLI_ARG_RESOLVE_IP_HTTP       },
+    { "RESOLVE_IP_HTTPS",      FWKNOP_CLI_ARG_RESOLVE_IP_HTTPS      },
+    { "RESOLVE_HTTP_ONLY",     FWKNOP_CLI_ARG_RESOLVE_HTTP_ONLY     },
+    { "WGET_CMD",              FWKNOP_CLI_ARG_WGET_CMD              },
+    { "NO_SAVE_ARGS",          FWKNOP_CLI_ARG_NO_SAVE_ARGS          }
 };
 
 /* Array to define which conf. variables are critical and should not be
@@ -852,6 +862,7 @@ create_fwknoprc(const char *rcfile)
         "#GPG_EXE             /path/to/gpg\n"
         "#GPG_SIGNER          <signer ID>\n"
         "#GPG_RECIPIENT       <recipient ID>\n"
+        "#NO_SAVE_ARGS        N\n"
         "\n"
         "# User-provided named stanzas:\n"
         "\n"
@@ -946,14 +957,14 @@ parse_rc_param(fko_cli_options_t *options, const char *var_name, char * val)
     {
         /* In case this was set previously
         */
-        options->resolve_ip_http = 0;
+        options->resolve_ip_http_https = 0;
 
         /* use source, resolve, or an actual IP
         */
         if(strcasecmp(val, "source") == 0)
             strlcpy(options->allow_ip_str, "0.0.0.0", sizeof(options->allow_ip_str));
         else if(strcasecmp(val, "resolve") == 0)
-            options->resolve_ip_http = 1;
+            options->resolve_ip_http_https = 1;
         else /* Assume IP address and validate */
         {
             strlcpy(options->allow_ip_str, val, sizeof(options->allow_ip_str));
@@ -1138,6 +1149,12 @@ parse_rc_param(fko_cli_options_t *options, const char *var_name, char * val)
         if (is_yes_str(val))
             options->use_hmac = 1;
     }
+    /* --use-wget-user-agent */
+    else if (var->pos == FWKNOP_CLI_ARG_USE_WGET_USER_AGENT)
+    {
+        if (is_yes_str(val))
+            options->use_wget_user_agent = 1;
+    }
     /* Key file */
     else if (var->pos == FWKNOP_CLI_ARG_KEY_FILE)
     {
@@ -1173,6 +1190,20 @@ parse_rc_param(fko_cli_options_t *options, const char *var_name, char * val)
         }
         strlcpy(options->resolve_url, val, tmpint);
     }
+    /* wget command */
+    else if (var->pos == FWKNOP_CLI_ARG_WGET_CMD)
+    {
+        if(options->wget_bin != NULL)
+            free(options->wget_bin);
+        tmpint = strlen(val)+1;
+        options->wget_bin = calloc(1, tmpint);
+        if(options->wget_bin == NULL)
+        {
+            log_msg(LOG_VERBOSITY_ERROR,"Memory allocation error for wget command path.");
+            exit(EXIT_FAILURE);
+        }
+        strlcpy(options->wget_bin, val, tmpint);
+    }
     /* NAT Local ? */
     else if (var->pos == FWKNOP_CLI_ARG_NAT_LOCAL)
     {
@@ -1213,11 +1244,35 @@ parse_rc_param(fko_cli_options_t *options, const char *var_name, char * val)
         if (parse_error == 0)
             log_set_verbosity(LOG_DEFAULT_VERBOSITY + options->verbose);
     }
-    /* RESOLVE_IP_HTTP ? */
+    /* RESOLVE_IP_HTTPS ? */
+    else if (var->pos == FWKNOP_CLI_ARG_RESOLVE_IP_HTTPS)
+    {
+        if (is_yes_str(val))
+            options->resolve_ip_http_https = 1;
+        else;
+    }
+    /* RESOLVE_IP_HTTP ? This actually results in HTTPS resolution by default
+     * unless --resolve-http-only is also given
+    */
     else if (var->pos == FWKNOP_CLI_ARG_RESOLVE_IP_HTTP)
     {
         if (is_yes_str(val))
-            options->resolve_ip_http = 1;
+            options->resolve_ip_http_https = 1;
+        else;
+    }
+    /* RESOLVE_HTTP_ONLY ?  Force HTTP instead of HTTPS IP resolution.
+    */
+    else if (var->pos == FWKNOP_CLI_ARG_RESOLVE_HTTP_ONLY)
+    {
+        if (is_yes_str(val))
+            options->resolve_http_only = 1;
+        else;
+    }
+    /* avoid saving .fwknop.run by default */
+    else if (var->pos == FWKNOP_CLI_ARG_NO_SAVE_ARGS)
+    {
+        if (is_yes_str(val))
+            options->no_save_args = 1;
         else;
     }
     /* The variable is not a configuration variable */
@@ -1343,6 +1398,9 @@ add_single_var_to_rc(FILE* fhandle, short var_pos, fko_cli_options_t *options)
         case FWKNOP_CLI_ARG_USE_HMAC :
             bool_to_yesno(options->use_hmac, val, sizeof(val));
             break;
+        case FWKNOP_CLI_ARG_USE_WGET_USER_AGENT :
+            bool_to_yesno(options->use_wget_user_agent, val, sizeof(val));
+            break;
         case FWKNOP_CLI_ARG_NAT_ACCESS :
             strlcpy(val, options->nat_access_str, sizeof(val));
             break;
@@ -1369,8 +1427,21 @@ add_single_var_to_rc(FILE* fhandle, short var_pos, fko_cli_options_t *options)
             else
                 snprintf(val, sizeof(val)-1, "%d", options->verbose);
             break;
+        case FWKNOP_CLI_ARG_RESOLVE_IP_HTTPS:
+            bool_to_yesno(options->resolve_ip_http_https, val, sizeof(val));
+            break;
         case FWKNOP_CLI_ARG_RESOLVE_IP_HTTP:
-            bool_to_yesno(options->resolve_ip_http, val, sizeof(val));
+            bool_to_yesno(options->resolve_ip_http_https, val, sizeof(val));
+            break;
+        case FWKNOP_CLI_ARG_RESOLVE_HTTP_ONLY:
+            bool_to_yesno(options->resolve_http_only, val, sizeof(val));
+            break;
+        case FWKNOP_CLI_ARG_WGET_CMD :
+            if (options->wget_bin != NULL)
+                strlcpy(val, options->wget_bin, sizeof(val));
+            break;
+        case FWKNOP_CLI_ARG_NO_SAVE_ARGS :
+            bool_to_yesno(options->no_save_args, val, sizeof(val));
             break;
         default:
             log_msg(LOG_VERBOSITY_WARNING, "Warning from add_single_var_to_rc() : Bad variable position %u", var->pos);
@@ -1737,9 +1808,9 @@ validate_options(fko_cli_options_t *options)
         }
 
         if (options->resolve_url != NULL)
-            options->resolve_ip_http = 1;
+            options->resolve_ip_http_https = 1;
 
-        if (!options->resolve_ip_http)
+        if (!options->resolve_ip_http_https)
         {
             if(options->allow_ip_str[0] == 0x0)
             {
@@ -1780,7 +1851,7 @@ validate_options(fko_cli_options_t *options)
         }
     }
 
-    if(options->resolve_ip_http || options->spa_proto == FKO_PROTO_HTTP)
+    if(options->resolve_ip_http_https || options->spa_proto == FKO_PROTO_HTTP)
         if (options->http_user_agent[0] == '\0')
             snprintf(options->http_user_agent, HTTP_MAX_USER_AGENT_LEN,
                 "%s%s", "Fwknop/", MY_VERSION);
@@ -1859,7 +1930,7 @@ set_defaults(fko_cli_options_t *options)
 void
 config_init(fko_cli_options_t *options, int argc, char **argv)
 {
-    int                 cmd_arg, index, is_err;
+    int                 cmd_arg, index, is_err, rlen=0;
     fko_var_bitmask_t   var_bitmask;
     char                rcfile[MAX_PATH_LEN] = {0};
 
@@ -1967,6 +2038,15 @@ config_init(fko_cli_options_t *options, int argc, char **argv)
                 }
                 add_var_to_bitmask(FWKNOP_CLI_ARG_FW_TIMEOUT, &var_bitmask);
                 break;
+            case FAULT_INJECTION_TAG:
+#if HAVE_LIBFIU
+                strlcpy(options->fault_injection_tag, optarg, sizeof(options->fault_injection_tag));
+#else
+                log_msg(LOG_VERBOSITY_ERROR,
+                    "fwknop not compiled with fault injection support.", optarg);
+                exit(EXIT_FAILURE);
+#endif
+                break;
             case 'g':
             case GPG_ENCRYPTION:
                 options->use_gpg = 1;
@@ -2116,6 +2196,7 @@ config_init(fko_cli_options_t *options, int argc, char **argv)
                 break;
             case NO_SAVE_ARGS:
                 options->no_save_args = 1;
+                add_var_to_bitmask(FWKNOP_CLI_ARG_NO_SAVE_ARGS, &var_bitmask);
                 break;
             case 'n':
                 /* We already handled this earlier, so we do nothing here
@@ -2150,21 +2231,41 @@ config_init(fko_cli_options_t *options, int argc, char **argv)
                 add_var_to_bitmask(FWKNOP_CLI_ARG_RAND_PORT, &var_bitmask);
                 break;
             case 'R':
-                options->resolve_ip_http = 1;
-                add_var_to_bitmask(FWKNOP_CLI_ARG_RESOLVE_IP_HTTP, &var_bitmask);
+                options->resolve_ip_http_https = 1;
+                add_var_to_bitmask(FWKNOP_CLI_ARG_RESOLVE_IP_HTTPS, &var_bitmask);
+                break;
+            case RESOLVE_HTTP_ONLY:
+                options->resolve_http_only = 1;
+                options->resolve_ip_http_https = 1;
+                add_var_to_bitmask(FWKNOP_CLI_ARG_RESOLVE_HTTP_ONLY, &var_bitmask);
+                add_var_to_bitmask(FWKNOP_CLI_ARG_RESOLVE_IP_HTTPS, &var_bitmask);
                 break;
             case RESOLVE_URL:
                 if(options->resolve_url != NULL)
                     free(options->resolve_url);
-                options->resolve_url = calloc(1, strlen(optarg)+1);
+                rlen = strlen(optarg) + 1;
+                options->resolve_url = calloc(1, rlen);
                 if(options->resolve_url == NULL)
                 {
                     log_msg(LOG_VERBOSITY_ERROR, "Memory allocation error for resolve URL.");
                     exit(EXIT_FAILURE);
                 }
-                strlcpy(options->resolve_url, optarg, strlen(optarg)+1);
+                strlcpy(options->resolve_url, optarg, rlen);
                 add_var_to_bitmask(FWKNOP_CLI_ARG_RESOLVE_URL, &var_bitmask);
                 break;
+            case 'w':
+                if(options->wget_bin != NULL)
+                    free(options->wget_bin);
+                rlen = strlen(optarg) + 1;
+                options->wget_bin = calloc(1, rlen);
+                if(options->wget_bin == NULL)
+                {
+                    log_msg(LOG_VERBOSITY_ERROR, "Memory allocation error for resolve URL.");
+                    exit(EXIT_FAILURE);
+                }
+                strlcpy(options->wget_bin, optarg, rlen);
+                add_var_to_bitmask(FWKNOP_CLI_ARG_WGET_CMD, &var_bitmask);
+                break;
             case SHOW_LAST_ARGS:
                 options->show_last_command = 1;
                 break;
@@ -2269,6 +2370,10 @@ config_init(fko_cli_options_t *options, int argc, char **argv)
                 add_var_to_bitmask(FWKNOP_CLI_ARG_USE_HMAC, &var_bitmask);
                 options->use_hmac = 1;
                 break;
+            case USE_WGET_USER_AGENT:
+                add_var_to_bitmask(FWKNOP_CLI_ARG_USE_WGET_USER_AGENT, &var_bitmask);
+                options->use_wget_user_agent = 1;
+                break;
             case FORCE_SAVE_RC_STANZA:
                 options->force_save_rc_stanza = 1;
                 break;
@@ -2332,7 +2437,7 @@ usage(void)
       "                             '$HOME/.fwknoprc' file to provide some of all\n"
       "                             of the configuration parameters.\n"
       "                             If more arguments are set through the command\n"
-      "                             line, the configuration is updated accordingly\n"
+      "                             line, the configuration is updated accordingly.\n"
       " -A, --access                Provide a list of ports/protocols to open\n"
       "                             on the server (e.g. 'tcp/22').\n"
       " -a, --allow-ip              Specify IP address to allow within the SPA\n"
@@ -2361,16 +2466,27 @@ usage(void)
       "                             fwknopd server can ignore such requests).\n"
       " -S, --source-port           Set the source port for outgoing SPA packet.\n"
       " -Q, --spoof-source          Set the source IP for outgoing SPA packet.\n"
-      " -R, --resolve-ip-http       Resolve the external network IP by\n"
+      " -R, --resolve-ip-https      Resolve the external network IP by\n"
       "                             connecting to a URL such as the default of:\n"
-      "                             http://" HTTP_RESOLVE_HOST HTTP_RESOLVE_URL "\n"
-      "                             This can be overridden with the --resolve-url\n"
-      "                             option.\n"
+      "                             https://" HTTP_RESOLVE_HOST HTTP_RESOLVE_URL "\n"
+      "                             with wget in --secure-protocol mode (SSL).\n"
+      "                             The URL can be overridden with the\n"
+      "                             --resolve-url option.\n"
+      "     --resolve-http-only     Force external IP resolution via HTTP instead\n"
+      "                             HTTPS (via the URL mentioned above). This is\n"
+      "                             not recommended since it would open fwknop\n"
+      "                             to potential MITM attacks if the IP resolution\n"
+      "                             HTTP connection is altered en-route by a third\n"
+      "                             party.\n"
       "     --resolve-url           Override the default URL used for resolving\n"
       "                             the source IP address.\n"
       " -u, --user-agent            Set the HTTP User-Agent for resolving the\n"
       "                             external IP via -R, or for sending SPA\n"
-      "                             packets over HTTP.\n"
+      "                             packets over HTTP. The default is\n"
+      "                             Fwknop/<version> if this option is not used.\n"
+      "     --use-wget-user-agent   Use the default wget User-Agent string instead\n"
+      "                             of Fwknop/<version>.\n"
+      " -w, --wget-cmd              Manually set the path to wget in -R mode.\n"
       " -H, --http-proxy            Specify an HTTP proxy host through which the\n"
       "                             SPA packet will be sent.  The port can also be\n"
       "                             specified here by following the host/ip with\n"
@@ -2387,9 +2503,9 @@ usage(void)
       " -K, --key-gen-file          Write generated Rijndael + HMAC keys to a\n"
       "                             file\n"
       "     --key-rijndael          Specify the Rijndael key. Since the password is\n"
-      "                             visible to utilities (like 'ps' under Unix) this\n"
-      "                             form should only be used where security is not\n"
-      "                             important.\n"
+      "                             visible to utilities (like 'ps' under Unix)\n"
+      "                             this form should only be used where security is\n"
+      "                             not important.\n"
       "     --key-base64-rijndael   Specify the base64 encoded Rijndael key. Since\n"
       "                             the password is visible to utilities (like 'ps'\n"
       "                             under Unix) this form should only be used where\n"
