@@ -131,8 +131,7 @@ _rijndael_encrypt(fko_ctx_t ctx, const char *enc_key, const int enc_key_len)
         zero_free_rv = zero_free(ctx->encrypted_msg,
                 strnlen(ctx->encrypted_msg, MAX_SPA_ENCODED_MSG_SIZE));
 
-    ctx->encrypted_msg     = strdup(b64ciphertext);
-    ctx->encrypted_msg_len = strnlen(ctx->encrypted_msg, MAX_SPA_ENCODED_MSG_SIZE);
+    ctx->encrypted_msg = strdup(b64ciphertext);
 
     /* Clean-up
     */
@@ -149,6 +148,8 @@ _rijndael_encrypt(fko_ctx_t ctx, const char *enc_key, const int enc_key_len)
     if(ctx->encrypted_msg == NULL)
         return(FKO_ERROR_MEMORY_ALLOCATION);
 
+    ctx->encrypted_msg_len = strnlen(ctx->encrypted_msg, MAX_SPA_ENCODED_MSG_SIZE);
+
     if(! is_valid_encoded_msg_len(ctx->encrypted_msg_len))
         return(FKO_ERROR_INVALID_DATA_ENCRYPT_RESULT_MSGLEN_VALIDFAIL);
 
@@ -377,8 +378,7 @@ gpg_encrypt(fko_ctx_t ctx, const char *enc_key)
         zero_free_rv = zero_free(ctx->encrypted_msg,
                 strnlen(ctx->encrypted_msg, MAX_SPA_ENCODED_MSG_SIZE));
 
-    ctx->encrypted_msg     = strdup(b64cipher);
-    ctx->encrypted_msg_len = strnlen(ctx->encrypted_msg, MAX_SPA_ENCODED_MSG_SIZE);
+    ctx->encrypted_msg = strdup(b64cipher);
 
     /* Clean-up
     */
@@ -395,6 +395,8 @@ gpg_encrypt(fko_ctx_t ctx, const char *enc_key)
     if(ctx->encrypted_msg == NULL)
         return(FKO_ERROR_MEMORY_ALLOCATION);
 
+    ctx->encrypted_msg_len = strnlen(ctx->encrypted_msg, MAX_SPA_ENCODED_MSG_SIZE);
+
     if(! is_valid_encoded_msg_len(ctx->encrypted_msg_len))
         return(FKO_ERROR_INVALID_DATA_ENCRYPT_GPG_RESULT_MSGLEN_VALIDFAIL);
 
@@ -478,11 +480,19 @@ gpg_decrypt(fko_ctx_t ctx, const char *dec_key)
 int
 fko_set_spa_encryption_type(fko_ctx_t ctx, const short encrypt_type)
 {
+#if HAVE_LIBFIU
+    fiu_return_on("fko_set_spa_encryption_type_init",
+            FKO_ERROR_CTX_NOT_INITIALIZED);
+#endif
     /* Must be initialized
     */
     if(!CTX_INITIALIZED(ctx))
         return(FKO_ERROR_CTX_NOT_INITIALIZED);
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_set_spa_encryption_type_val",
+            FKO_ERROR_INVALID_DATA_ENCRYPT_TYPE_VALIDFAIL);
+#endif
     if(encrypt_type < 0 || encrypt_type >= FKO_LAST_ENCRYPTION_TYPE)
         return(FKO_ERROR_INVALID_DATA_ENCRYPT_TYPE_VALIDFAIL);
 
@@ -513,11 +523,19 @@ fko_get_spa_encryption_type(fko_ctx_t ctx, short *enc_type)
 int
 fko_set_spa_encryption_mode(fko_ctx_t ctx, const int encrypt_mode)
 {
+#if HAVE_LIBFIU
+    fiu_return_on("fko_set_spa_encryption_mode_init",
+            FKO_ERROR_CTX_NOT_INITIALIZED);
+#endif
     /* Must be initialized
     */
     if(!CTX_INITIALIZED(ctx))
         return(FKO_ERROR_CTX_NOT_INITIALIZED);
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_set_spa_encryption_mode_val",
+            FKO_ERROR_INVALID_DATA_ENCRYPT_MODE_VALIDFAIL);
+#endif
     if(encrypt_mode < 0 || encrypt_mode >= FKO_LAST_ENC_MODE)
         return(FKO_ERROR_INVALID_DATA_ENCRYPT_MODE_VALIDFAIL);
 
