@@ -65,6 +65,8 @@ zero_cmd_buffers(void)
     memset(cmd_out, 0x0, STANDARD_CMD_OUT_BUFSIZE);
 }
 
+static int pid_status = 0;
+
 static int
 ipfw_set_exists(const fko_srv_options_t *opts,
     const char *fw_command, const unsigned short set_num)
@@ -78,7 +80,8 @@ ipfw_set_exists(const fko_srv_options_t *opts,
         set_num
     );
 
-    res = run_extcmd(cmd_buf, cmd_out, STANDARD_CMD_OUT_BUFSIZE, 0);
+    res = run_extcmd(cmd_buf, cmd_out, STANDARD_CMD_OUT_BUFSIZE,
+            WANT_STDERR, NO_TIMEOUT, &pid_status, opts);
 
     log_msg(LOG_DEBUG, "ipfw_set_exists() CMD: '%s' (res: %d)",
         cmd_buf, res);
@@ -113,7 +116,8 @@ fw_dump_rules(const fko_srv_options_t * const opts)
             opts->fw_config->fw_command
         );
 
-        res = system(cmd_buf);
+        res = run_extcmd(cmd_buf, NULL, 0, NO_STDERR,
+                        NO_TIMEOUT, &pid_status, opts);
 
         log_msg(LOG_DEBUG, "fw_dump_rules() CMD: '%s' (res: %d)",
             cmd_buf, res);
@@ -140,7 +144,8 @@ fw_dump_rules(const fko_srv_options_t * const opts)
         );
 
         printf("\nActive Rules:\n");
-        res = system(cmd_buf);
+        res = run_extcmd(cmd_buf, NULL, 0, NO_STDERR,
+                    NO_TIMEOUT, &pid_status, opts);
 
         log_msg(LOG_DEBUG, "fw_dump_rules() CMD: '%s' (res: %d)",
             cmd_buf, res);
@@ -160,7 +165,8 @@ fw_dump_rules(const fko_srv_options_t * const opts)
         );
 
         printf("\nExpired Rules:\n");
-        res = system(cmd_buf);
+        res = run_extcmd(cmd_buf, NULL, 0, NO_STDERR,
+                    NO_TIMEOUT, &pid_status, opts);
 
         log_msg(LOG_DEBUG, "fw_dump_rules() CMD: '%s' (res: %d)",
             cmd_buf, res);
@@ -280,7 +286,8 @@ fw_initialize(const fko_srv_options_t * const opts)
             fwc.active_set_num
         );
 
-        res = run_extcmd(cmd_buf, err_buf, CMD_BUFSIZE, 0);
+        res = run_extcmd(cmd_buf, err_buf, CMD_BUFSIZE,
+                    WANT_STDERR, NO_TIMEOUT, &pid_status, opts);
 
         log_msg(LOG_DEBUG, "fw_initialize() CMD: '%s' (res: %d, err: %s)",
             cmd_buf, res, err_buf);
@@ -310,7 +317,8 @@ fw_initialize(const fko_srv_options_t * const opts)
             fwc.expire_set_num
         );
 
-        res = run_extcmd(cmd_buf, err_buf, CMD_BUFSIZE, 0);
+        res = run_extcmd(cmd_buf, err_buf, CMD_BUFSIZE,
+                    WANT_STDERR, NO_TIMEOUT, &pid_status, opts);
 
         log_msg(LOG_DEBUG, "fw_initialize() CMD: '%s' (res: %d, err: %s)",
             cmd_buf, res, err_buf);
@@ -332,7 +340,8 @@ fw_initialize(const fko_srv_options_t * const opts)
         fwc.expire_set_num
     );
 
-    res = run_extcmd(cmd_buf, cmd_out, STANDARD_CMD_OUT_BUFSIZE, 0);
+    res = run_extcmd(cmd_buf, cmd_out, STANDARD_CMD_OUT_BUFSIZE,
+                WANT_STDERR, NO_TIMEOUT, &pid_status, opts);
 
     log_msg(LOG_DEBUG, "fw_initialize() CMD: '%s' (res: %d)",
         cmd_buf, res);
@@ -413,7 +422,8 @@ fw_cleanup(const fko_srv_options_t * const opts)
             fwc.active_set_num
         );
 
-        res = system(cmd_buf);
+        res = run_extcmd(cmd_buf, NULL, 0, NO_STDERR,
+                    NO_TIMEOUT, &pid_status, opts);
 
         log_msg(LOG_DEBUG, "fw_cleanup() CMD: '%s' (res: %d)",
             cmd_buf, res);
@@ -526,7 +536,8 @@ process_spa_request(const fko_srv_options_t * const opts,
                 exp_ts
             );
 
-            res = run_extcmd(cmd_buf, err_buf, CMD_BUFSIZE, 0);
+            res = run_extcmd(cmd_buf, err_buf, CMD_BUFSIZE,
+                            WANT_STDERR, NO_TIMEOUT, &pid_status, opts);
 
             log_msg(LOG_DEBUG, "process_spa_request() CMD: '%s' (res: %d, err: %s)",
                 cmd_buf, res, err_buf);
@@ -620,7 +631,8 @@ check_firewall_rules(const fko_srv_options_t * const opts)
         fwc.active_set_num
     );
 
-    res = run_extcmd(cmd_buf, cmd_out, STANDARD_CMD_OUT_BUFSIZE, 0);
+    res = run_extcmd(cmd_buf, cmd_out, STANDARD_CMD_OUT_BUFSIZE,
+                WANT_STDERR, NO_TIMEOUT, &pid_status, opts);
 
     log_msg(LOG_DEBUG, "check_firewall_rules() CMD: '%s' (res: %d)",
         cmd_buf, res);
@@ -726,7 +738,8 @@ check_firewall_rules(const fko_srv_options_t * const opts)
                     fwc.expire_set_num
                 );
 
-                res = run_extcmd(cmd_buf, err_buf, CMD_BUFSIZE, 0);
+                res = run_extcmd(cmd_buf, err_buf, CMD_BUFSIZE,
+                            WANT_STDERR, NO_TIMEOUT, &pid_status, opts);
 
                 log_msg(LOG_DEBUG, "check_firewall_rules() CMD: '%s' (res: %d, err: %s)",
                     cmd_buf, res, err_buf);
@@ -794,7 +807,8 @@ ipfw_purge_expired_rules(const fko_srv_options_t *opts)
         fwc.expire_set_num
     );
 
-    res = run_extcmd(cmd_buf, cmd_out, STANDARD_CMD_OUT_BUFSIZE, 0);
+    res = run_extcmd(cmd_buf, cmd_out, STANDARD_CMD_OUT_BUFSIZE,
+            WANT_STDERR, NO_TIMEOUT, &pid_status, opts);
 
     log_msg(LOG_DEBUG, "ipfw_purge_expired_rules() CMD: '%s' (res: %d)",
         cmd_buf, res);
@@ -901,7 +915,8 @@ ipfw_purge_expired_rules(const fko_srv_options_t *opts)
             curr_rule
         );
 
-        res = run_extcmd(cmd_buf, cmd_out, STANDARD_CMD_OUT_BUFSIZE, 0);
+        res = run_extcmd(cmd_buf, cmd_out, STANDARD_CMD_OUT_BUFSIZE,
+                    WANT_STDERR, NO_TIMEOUT, &pid_status, opts);
 
         log_msg(LOG_DEBUG, "ipfw_purge_expired_rules() CMD: '%s' (res: %d)",
             cmd_buf, res);
