@@ -31,31 +31,49 @@
 #include "fwknopd.h"
 #include "access.h"
 #include "config_init.h"
-#include "process_packet.h"
-#include "pcap_capture.h"
 #include "log_msg.h"
 #include "utils.h"
 #include "fw_util.h"
 #include "sig_handler.h"
 #include "replay_cache.h"
 #include "tcp_server.h"
+#include "udp_server.h"
+
+#if USE_LIBPCAP
+  #include "pcap_capture.h"
+#endif
 
 /* Prototypes
 */
-static void check_dir_path(const char * const path,
+static int check_dir_path(const char * const path,
         const char * const path_name, const unsigned char use_basename);
 static int make_dir_path(const char * const path);
 static void daemonize_process(fko_srv_options_t * const opts);
+static int stop_fwknopd(fko_srv_options_t * const opts);
+static int status_fwknopd(fko_srv_options_t * const opts);
+static int restart_fwknopd(fko_srv_options_t * const opts);
 static int write_pid_file(fko_srv_options_t *opts);
+static int handle_signals(fko_srv_options_t *opts);
+static void setup_pid(fko_srv_options_t *opts);
+static void init_digest_cache(fko_srv_options_t *opts);
+static void set_locale(fko_srv_options_t *opts);
 static pid_t get_running_pid(const fko_srv_options_t *opts);
+#if AFL_FUZZING
+static void afl_pkt_from_stdin(fko_srv_options_t *opts);
+#endif
+
+#if HAVE_LIBFIU
+static void enable_fault_injections(fko_srv_options_t * const opts);
+#endif
+
+#if AFL_FUZZING
+#define AFL_MAX_PKT_SIZE  1024
+#define AFL_DUMP_CTX_SIZE 4096
+#endif
 
 int
 main(int argc, char **argv)
 {
-    int                 res, last_sig, rp_cache_count, is_err;
-    char               *locale;
-    pid_t               old_pid;
-
     fko_srv_options_t   opts;
 
     while(1)
@@ -64,119 +82,29 @@ main(int argc, char **argv)
         */
         config_init(&opts, argc, argv);
 
+#if HAVE_LIBFIU
+        /* Set any fault injection points early
+        */
+        enable_fault_injections(&opts);
+#endif
+
         /* Process any options that do their thing and exit.
         */
 
-        /* Kill the currently running fwknopd?
+        /* Kill the currently running fwknopd process?
         */
         if(opts.kill == 1)
-        {
-            old_pid = get_running_pid(&opts);
+            clean_exit(&opts, NO_FW_CLEANUP, stop_fwknopd(&opts));
 
-            if(old_pid > 0)
-            {
-                res    = kill(old_pid, SIGTERM);
-                is_err = kill(old_pid, 0);
-
-                if(res == 0 && is_err != 0)
-                {
-                    fprintf(stdout, "Killed fwknopd (pid=%i)\n", old_pid);
-                    clean_exit(&opts, NO_FW_CLEANUP, EXIT_SUCCESS);
-                }
-                else
-                {
-                    /* give a bit of time for process shutdown and check again
-                    */
-                    sleep(1);
-                    is_err = kill(old_pid, 0);
-                    if(is_err != 0)
-                    {
-                        fprintf(stdout, "Killed fwknopd (pid=%i) via SIGTERM\n",
-                                old_pid);
-                        clean_exit(&opts, NO_FW_CLEANUP, EXIT_SUCCESS);
-                    }
-                    else
-                    {
-                        res    = kill(old_pid, SIGKILL);
-                        is_err = kill(old_pid, 0);
-                        if(res == 0 && is_err != 0)
-                        {
-                            fprintf(stdout,
-                                    "Killed fwknopd (pid=%i) via SIGKILL\n",
-                                    old_pid);
-                            clean_exit(&opts, NO_FW_CLEANUP, EXIT_SUCCESS);
-                        }
-                        else
-                        {
-                            sleep(1);
-                            is_err = kill(old_pid, 0);
-                            if(is_err != 0)
-                            {
-                                fprintf(stdout,
-                                        "Killed fwknopd (pid=%i) via SIGKILL\n",
-                                        old_pid);
-                                clean_exit(&opts, NO_FW_CLEANUP, EXIT_SUCCESS);
-                            }
-                            else
-                            {
-                                perror("Unable to kill fwknop: ");
-                                clean_exit(&opts, NO_FW_CLEANUP, EXIT_FAILURE);
-                            }
-                        }
-                    }
-                }
-            }
-            else
-            {
-                fprintf(stderr, "No running fwknopd detected.\n");
-                clean_exit(&opts, NO_FW_CLEANUP, EXIT_FAILURE);
-            }
-        }
-
-        /* Status of the currently running fwknopd?
+        /* Status of the currently running fwknopd process?
         */
         if(opts.status == 1)
-        {
-            old_pid = write_pid_file(&opts);
+            clean_exit(&opts, NO_FW_CLEANUP, status_fwknopd(&opts));
 
-            if(old_pid > 0)
-            {
-                fprintf(stdout, "Detected fwknopd is running (pid=%i).\n", old_pid);
-                clean_exit(&opts, NO_FW_CLEANUP, EXIT_SUCCESS);
-            }
-            else
-            {
-                fprintf(stdout, "No running fwknopd detected.\n");
-                clean_exit(&opts, NO_FW_CLEANUP, EXIT_FAILURE);
-            }
-        }
-
-        /* Restart the currently running fwknopd?
+        /* Restart the currently running fwknopd process?
         */
-        if(opts.restart == 1 || opts.status == 1)
-        {
-            old_pid = get_running_pid(&opts);
-
-            if(old_pid > 0)
-            {
-                res = kill(old_pid, SIGHUP);
-                if(res == 0)
-                {
-                    fprintf(stdout, "Sent restart signal to fwknopd (pid=%i)\n", old_pid);
-                    clean_exit(&opts, NO_FW_CLEANUP, EXIT_SUCCESS);
-                }
-                else
-                {
-                    perror("Unable to send signal to fwknop: ");
-                    clean_exit(&opts, NO_FW_CLEANUP, EXIT_FAILURE);
-                }
-            }
-            else
-            {
-                fprintf(stdout, "No running fwknopd detected.\n");
-                clean_exit(&opts, NO_FW_CLEANUP, EXIT_FAILURE);
-            }
-        }
+        if(opts.restart == 1)
+            clean_exit(&opts, NO_FW_CLEANUP, restart_fwknopd(&opts));
 
         /* Initialize logging.
         */
@@ -188,35 +116,19 @@ main(int argc, char **argv)
 #if HAVE_LOCALE_H
         /* Set the locale if specified.
         */
-        if(opts.config[CONF_LOCALE] != NULL
-          && strncasecmp(opts.config[CONF_LOCALE], "NONE", 4) != 0)
-        {
-            locale = setlocale(LC_ALL, opts.config[CONF_LOCALE]);
-
-            if(locale == NULL)
-            {
-                log_msg(LOG_ERR,
-                    "WARNING: Unable to set locale to '%s'.",
-                    opts.config[CONF_LOCALE]
-                );
-            }
-            else
-            {
-                log_msg(LOG_INFO,
-                    "Locale set to '%s'.", opts.config[CONF_LOCALE]
-                );
-            }
-        }
+        set_locale(&opts);
 #endif
 
         /* Make sure we have a valid run dir and path leading to digest file
          * in case it configured to be somewhere other than the run dir.
         */
-        check_dir_path((const char *)opts.config[CONF_FWKNOP_RUN_DIR], "Run", 0);
+        if(!opts.afl_fuzzing
+                && ! check_dir_path((const char *)opts.config[CONF_FWKNOP_RUN_DIR], "Run", 0))
+            clean_exit(&opts, NO_FW_CLEANUP, EXIT_FAILURE);
 
         /* Initialize the firewall rules handler based on the fwknopd.conf
          * file, but (for iptables firewalls) don't flush any rules or create
-         * any chains yet.  This allows us to dump the current firewall rules
+         * any chains yet. This allows us to dump the current firewall rules
          * via fw_rules_dump() in --fw-list mode before changing around any rules
          * of an existing fwknopd process.
         */
@@ -249,42 +161,10 @@ main(int argc, char **argv)
             clean_exit(&opts, NO_FW_CLEANUP, EXIT_SUCCESS);
         }
 
-        /* If we are a new process (just being started), proceed with normal
-         * start-up.  Otherwise, we are here as a result of a signal sent to an
-         * existing process and we want to restart.
+        /* Acquire pid, become a daemon or run in the foreground, write pid
+         * to pid file.
         */
-        if(get_running_pid(&opts) != getpid())
-        {
-            /* If foreground mode is not set, then fork off and become a daemon.
-            * Otherwise, attempt to get the pid file lock and go on.
-            */
-            if(opts.foreground == 0)
-            {
-                daemonize_process(&opts);
-            }
-            else
-            {
-                old_pid = write_pid_file(&opts);
-                if(old_pid > 0)
-                {
-                    fprintf(stderr,
-                        "[*] An instance of fwknopd is already running: (PID=%i).\n", old_pid
-                    );
-
-                    clean_exit(&opts, NO_FW_CLEANUP, EXIT_FAILURE);
-                }
-                else if(old_pid < 0)
-                {
-                    fprintf(stderr, "[*] PID file error. The lock may not be effective.\n");
-                }
-            }
-
-            log_msg(LOG_INFO, "Starting %s", MY_NAME);
-        }
-        else
-        {
-            log_msg(LOG_INFO, "Re-starting %s", MY_NAME);
-        }
+        setup_pid(&opts);
 
         if(opts.verbose > 1 && opts.foreground)
         {
@@ -296,107 +176,67 @@ main(int argc, char **argv)
          * with dbm support or with the default simple cache file strategy)
          * if so configured.
         */
-        if(strncasecmp(opts.config[CONF_ENABLE_DIGEST_PERSISTENCE], "Y", 1) == 0)
+        init_digest_cache(&opts);
+
+        if(opts.exit_after_parse_config)
         {
-            rp_cache_count = replay_cache_init(&opts);
-
-            if(rp_cache_count < 0)
-            {
-                log_msg(LOG_WARNING,
-                    "Error opening digest cache file. Incoming digests will not be remembered."
-                );
-                /* Destination points to heap memory, and is guaranteed to be
-                 * at least two bytes large via validate_options(),
-                 * DEF_ENABLE_DIGEST_PERSISTENCE, and set_config_entry()
-                */
-                strlcpy(opts.config[CONF_ENABLE_DIGEST_PERSISTENCE], "N", 2);
-            }
-
-            if(opts.verbose)
-                log_msg(LOG_ERR,
-                    "Using Digest Cache: '%s' (entry count = %i)",
-#if USE_FILE_CACHE
-                    opts.config[CONF_DIGEST_FILE], rp_cache_count
-#else
-                    opts.config[CONF_DIGEST_DB_FILE], rp_cache_count
-#endif
-                );
+            log_msg(LOG_INFO, "Configs parsed, exiting.");
+            clean_exit(&opts, NO_FW_CLEANUP, EXIT_SUCCESS);
         }
 
+#if AFL_FUZZING
+        /* SPA data from STDIN. */
+        if(opts.afl_fuzzing)
+            afl_pkt_from_stdin(&opts);
+#endif
+
         /* Prepare the firewall - i.e. flush any old rules and (for iptables)
          * create fwknop chains.
         */
-        if(fw_initialize(&opts) != 1)
+        if(!opts.test && (fw_initialize(&opts) != 1))
             clean_exit(&opts, FW_CLEANUP, EXIT_FAILURE);
 
-        /* If the TCP server option was set, fire it up here.
+        /* If we are to acquire SPA data via a UDP socket, start it up here.
+        */
+        if(opts.enable_udp_server ||
+                strncasecmp(opts.config[CONF_ENABLE_UDP_SERVER], "Y", 1) == 0)
+        {
+            if(run_udp_server(&opts) < 0)
+            {
+                log_msg(LOG_ERR, "Fatal run_udp_server() error");
+                clean_exit(&opts, FW_CLEANUP, EXIT_FAILURE);
+            }
+            else
+            {
+                break;
+            }
+        }
+
+        /* If the TCP server option was set, fire it up here. Note that in
+         * this mode, fwknopd still acquires SPA packets via libpcap. If you
+         * want to use UDP only without the libpcap dependency, see the FIXME...
         */
         if(strncasecmp(opts.config[CONF_ENABLE_TCP_SERVER], "Y", 1) == 0)
         {
-
-            strtol_wrapper(opts.config[CONF_TCPSERV_PORT],
-                    1, MAX_PORT, NO_EXIT_UPON_ERR, &is_err);
-            if(is_err == FKO_SUCCESS)
+            if(run_tcp_server(&opts) < 0)
             {
-                run_tcp_server(&opts);
-            }
-            else
-            {
-                log_msg(LOG_WARNING,
-                    "WARNING: ENABLE_TCP_SERVER is set, but TCPSERV_PORT is not valid. TCP server not started!"
-                );
+                log_msg(LOG_ERR, "Fatal run_tcp_server() error");
+                clean_exit(&opts, FW_CLEANUP, EXIT_FAILURE);
             }
         }
 
+#if USE_LIBPCAP
         /* Intiate pcap capture mode...
         */
-        pcap_capture(&opts);
+        if(strncasecmp(opts.config[CONF_ENABLE_UDP_SERVER], "N", 1) == 0)
+            pcap_capture(&opts);
+#endif
 
-        if(got_signal) {
-            last_sig   = got_signal;
-            got_signal = 0;
-
-            if(got_sighup)
-            {
-                log_msg(LOG_WARNING, "Got SIGHUP.  Re-reading configs.");
-                free_configs(&opts);
-                kill(opts.tcp_server_pid, SIGTERM);
-                usleep(1000000);
-                got_sighup = 0;
-            }
-            else if(got_sigint)
-            {
-                log_msg(LOG_WARNING, "Got SIGINT.  Exiting...");
-                got_sigint = 0;
-                break;
-            }
-            else if(got_sigterm)
-            {
-                log_msg(LOG_WARNING, "Got SIGTERM.  Exiting...");
-                got_sigterm = 0;
-                break;
-            }
-            else
-            {
-                log_msg(LOG_WARNING,
-                    "Got signal %i. No defined action but to exit.", last_sig);
-                break;
-            }
-        }
-        else if (opts.packet_ctr_limit > 0
-            && opts.packet_ctr >= opts.packet_ctr_limit)
-        {
-            log_msg(LOG_INFO,
-                "Packet count limit (%d) reached.  Exiting...",
-                opts.packet_ctr_limit);
+        /* Deal with any signals that we've received and break out
+         * of the loop for any terminating signals
+        */
+        if(handle_signals(&opts) == 1)
             break;
-        }
-        else    /* got_signal was not set (should be if we are here) */
-        {
-            log_msg(LOG_WARNING,
-                "Capture ended without signal.  Exiting...");
-            break;
-        }
     }
 
     log_msg(LOG_INFO, "Shutting Down fwknopd.");
@@ -411,30 +251,343 @@ main(int argc, char **argv)
         kill(opts.tcp_server_pid, SIGTERM);
 
         /* --DSS XXX: This seems to be necessary if the tcp server
-         *            was restarted by this program.  We need to
+         *            was restarted by this program. We need to
          *            investigate and fix this. For now, this works
          *            (it is kludgy, but does no harm afaik).
         */
         kill(opts.tcp_server_pid, SIGKILL);
     }
 
-    /* Other cleanup.
-    */
-    fw_cleanup(&opts);
-    free_logging();
+    clean_exit(&opts, FW_CLEANUP, EXIT_SUCCESS);
 
-#if USE_FILE_CACHE
-    free_replay_list(&opts);
-#endif
-
-    free_configs(&opts);
-
-    return(0);
+    return(EXIT_SUCCESS);  /* This never gets called */
 }
 
-/* Ensure the specified directory exists.  If not, create it or die.
+static void set_locale(fko_srv_options_t *opts)
+{
+    char               *locale;
+
+    if(opts->config[CONF_LOCALE] != NULL
+      && strncasecmp(opts->config[CONF_LOCALE], "NONE", 4) != 0)
+    {
+        locale = setlocale(LC_ALL, opts->config[CONF_LOCALE]);
+
+        if(locale == NULL)
+        {
+            log_msg(LOG_ERR,
+                "WARNING: Unable to set locale to '%s'.",
+                opts->config[CONF_LOCALE]
+            );
+        }
+        else
+        {
+            log_msg(LOG_INFO,
+                "Locale set to '%s'.", opts->config[CONF_LOCALE]
+            );
+        }
+    }
+    return;
+}
+
+#if AFL_FUZZING
+static void afl_pkt_from_stdin(fko_srv_options_t *opts)
+{
+    FILE                *fp = NULL;
+    fko_ctx_t           decode_ctx = NULL;
+    unsigned char       spa_pkt[AFL_MAX_PKT_SIZE] = {0};
+    int                 res = 0, es = EXIT_SUCCESS;
+    char                dump_buf[AFL_DUMP_CTX_SIZE];
+
+    fp = fdopen(STDIN_FILENO, "r");
+    if(fp != NULL)
+    {
+        if(fgets((char *)spa_pkt, AFL_MAX_PKT_SIZE, fp) == NULL)
+        {
+            fclose(fp);
+            clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+        }
+
+        fclose(fp);
+
+        fko_new(&decode_ctx);
+
+        res = fko_set_encoded_data(decode_ctx, (char *) spa_pkt,
+                strlen((char *)spa_pkt), 0, FKO_DIGEST_SHA256);
+
+        if(res == FKO_SUCCESS)
+            res = fko_set_spa_data(decode_ctx, (const char *) spa_pkt);
+        if(res == FKO_SUCCESS)
+            res = fko_decode_spa_data(decode_ctx);
+        if(res == FKO_SUCCESS)
+            res = dump_ctx_to_buffer(decode_ctx, dump_buf, sizeof(dump_buf));
+        if(res == FKO_SUCCESS)
+            log_msg(LOG_INFO, "%s", dump_buf);
+
+        fko_destroy(decode_ctx);
+
+        if(res == FKO_SUCCESS)
+        {
+            log_msg(LOG_INFO, "SPA packet decode: %s", fko_errstr(res));
+            es = EXIT_SUCCESS;
+        }
+        else
+        {
+            log_msg(LOG_ERR, "Could not decode SPA packet: %s", fko_errstr(res));
+            es = EXIT_FAILURE;
+        }
+    }
+    else
+        log_msg(LOG_ERR, "Could not acquire SPA packet from stdin.");
+
+    clean_exit(opts, NO_FW_CLEANUP, es);
+}
+#endif
+
+static void init_digest_cache(fko_srv_options_t *opts)
+{
+    int     rp_cache_count;
+
+#if AFL_FUZZING
+    if(opts->afl_fuzzing)
+        return;
+#endif
+
+    if(strncasecmp(opts->config[CONF_ENABLE_DIGEST_PERSISTENCE], "Y", 1) == 0)
+    {
+        rp_cache_count = replay_cache_init(opts);
+
+        if(rp_cache_count < 0)
+        {
+            log_msg(LOG_WARNING,
+                "Error opening digest cache file. Incoming digests will not be remembered."
+            );
+            /* Destination points to heap memory, and is guaranteed to be
+             * at least two bytes large via validate_options(),
+             * DEF_ENABLE_DIGEST_PERSISTENCE, and set_config_entry()
+            */
+            strlcpy(opts->config[CONF_ENABLE_DIGEST_PERSISTENCE], "N", 2);
+        }
+
+        if(opts->verbose)
+            log_msg(LOG_ERR,
+                "Using Digest Cache: '%s' (entry count = %i)",
+#if USE_FILE_CACHE
+                opts->config[CONF_DIGEST_FILE], rp_cache_count
+#else
+                opts->config[CONF_DIGEST_DB_FILE], rp_cache_count
+#endif
+            );
+    }
+    return;
+}
+
+static void setup_pid(fko_srv_options_t *opts)
+{
+    pid_t    old_pid;
+
+#if AFL_FUZZING
+    if(opts->afl_fuzzing)
+        return;
+#endif
+
+    /* If we are a new process (just being started), proceed with normal
+     * start-up. Otherwise, we are here as a result of a signal sent to an
+     * existing process and we want to restart.
+    */
+    if(get_running_pid(opts) != getpid())
+    {
+        /* If foreground mode is not set, then fork off and become a daemon.
+        * Otherwise, attempt to get the pid file lock and go on.
+        */
+        if(opts->foreground == 0)
+        {
+            daemonize_process(opts);
+        }
+        else
+        {
+            old_pid = write_pid_file(opts);
+            if(old_pid > 0)
+            {
+                fprintf(stderr,
+                    "[*] An instance of fwknopd is already running: (PID=%i).\n", old_pid
+                );
+
+                clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+            }
+            else if(old_pid < 0)
+            {
+                fprintf(stderr, "[*] PID file error. The lock may not be effective.\n");
+            }
+        }
+
+        log_msg(LOG_INFO, "Starting %s", MY_NAME);
+    }
+    else
+    {
+        log_msg(LOG_INFO, "Re-starting %s", MY_NAME);
+    }
+
+    return;
+}
+
+static int restart_fwknopd(fko_srv_options_t * const opts)
+{
+    int      res = 0;
+    pid_t    old_pid;
+
+    old_pid = get_running_pid(opts);
+
+    if(old_pid > 0)
+    {
+        res = kill(old_pid, SIGHUP);
+        if(res == 0)
+        {
+            fprintf(stdout, "Sent restart signal to fwknopd (pid=%i)\n", old_pid);
+            return EXIT_SUCCESS;
+        }
+        else
+        {
+            perror("Unable to send signal to fwknop: ");
+            return EXIT_FAILURE;
+        }
+    }
+
+    fprintf(stdout, "No running fwknopd detected.\n");
+    return EXIT_FAILURE;
+}
+
+static int status_fwknopd(fko_srv_options_t * const opts)
+{
+    pid_t    old_pid;
+
+    old_pid = write_pid_file(opts);
+
+    if(old_pid > 0)
+    {
+        fprintf(stdout, "Detected fwknopd is running (pid=%i).\n", old_pid);
+        return EXIT_SUCCESS;
+    }
+
+    fprintf(stdout, "No running fwknopd detected.\n");
+    return EXIT_FAILURE;
+}
+
+static int handle_signals(fko_srv_options_t *opts)
+{
+    int      last_sig = 0, rv = 1;
+
+    if(got_signal) {
+        last_sig   = got_signal;
+        got_signal = 0;
+
+        if(got_sighup)
+        {
+            log_msg(LOG_WARNING, "Got SIGHUP. Re-reading configs.");
+            free_configs(opts);
+            kill(opts->tcp_server_pid, SIGTERM);
+            usleep(1000000);
+            got_sighup = 0;
+            rv = 0;  /* this means fwknopd will not exit */
+        }
+        else if(got_sigint)
+        {
+            log_msg(LOG_WARNING, "Got SIGINT. Exiting...");
+            got_sigint = 0;
+        }
+        else if(got_sigterm)
+        {
+            log_msg(LOG_WARNING, "Got SIGTERM. Exiting...");
+            got_sigterm = 0;
+        }
+        else
+        {
+            log_msg(LOG_WARNING,
+                "Got signal %i. No defined action but to exit.", last_sig);
+        }
+    }
+    else if (opts->packet_ctr_limit > 0
+        && opts->packet_ctr >= opts->packet_ctr_limit)
+    {
+        log_msg(LOG_INFO,
+            "Packet count limit (%d) reached. Exiting...",
+            opts->packet_ctr_limit);
+    }
+    else    /* got_signal was not set (should be if we are here) */
+    {
+        log_msg(LOG_WARNING,
+            "Capture ended without signal. Exiting...");
+    }
+    return rv;
+}
+
+static int stop_fwknopd(fko_srv_options_t * const opts)
+{
+    int      res = 0, is_err = 0;
+    pid_t    old_pid;
+
+    old_pid = get_running_pid(opts);
+
+    if(old_pid > 0)
+    {
+        res    = kill(old_pid, SIGTERM);
+        is_err = kill(old_pid, 0);
+
+        if(res == 0 && is_err != 0)
+        {
+            fprintf(stdout, "Killed fwknopd (pid=%i)\n", old_pid);
+            return EXIT_SUCCESS;
+        }
+        else
+        {
+            /* give a bit of time for process shutdown and check again
+            */
+            sleep(1);
+            is_err = kill(old_pid, 0);
+            if(is_err != 0)
+            {
+                fprintf(stdout, "Killed fwknopd (pid=%i) via SIGTERM\n",
+                        old_pid);
+                return EXIT_SUCCESS;
+            }
+            else
+            {
+                res    = kill(old_pid, SIGKILL);
+                is_err = kill(old_pid, 0);
+                if(res == 0 && is_err != 0)
+                {
+                    fprintf(stdout,
+                            "Killed fwknopd (pid=%i) via SIGKILL\n",
+                            old_pid);
+                    return EXIT_SUCCESS;
+                }
+                else
+                {
+                    sleep(1);
+                    is_err = kill(old_pid, 0);
+                    if(is_err != 0)
+                    {
+                        fprintf(stdout,
+                                "Killed fwknopd (pid=%i) via SIGKILL\n",
+                                old_pid);
+                        return EXIT_SUCCESS;
+                    }
+                    else
+                    {
+                        perror("Unable to kill fwknop: ");
+                        return EXIT_FAILURE;
+                    }
+                }
+            }
+        }
+    }
+
+    fprintf(stderr, "No running fwknopd detected.\n");
+    return EXIT_FAILURE;
+}
+
+/* Ensure the specified directory exists. If not, create it or die.
 */
-static void
+static int
 check_dir_path(const char * const filepath, const char * const fp_desc, const unsigned char use_basename)
 {
     struct stat     st;
@@ -452,7 +605,7 @@ check_dir_path(const char * const filepath, const char * const fp_desc, const un
         log_msg(LOG_ERR,
             "Path '%s' is not absolute.", filepath
         );
-        exit(EXIT_FAILURE);
+        return 0;
     }
 
     /* If this is a file path that we want to use only the basename, strip
@@ -464,10 +617,10 @@ check_dir_path(const char * const filepath, const char * const fp_desc, const un
         strlcpy(tmp_path, filepath, sizeof(tmp_path));
 
     /* At this point, we should make the path is more than just the
-     * PATH_SEP.  If it is not, silently return.
+     * PATH_SEP. If it is not, silently return.
     */
     if(strlen(tmp_path) < 2)
-        return;
+        return 1;
 
     /* Make sure we have a valid directory.
     */
@@ -477,7 +630,7 @@ check_dir_path(const char * const filepath, const char * const fp_desc, const un
         if(errno == ENOENT)
         {
             log_msg(LOG_WARNING,
-                "%s directory: %s does not exist.  Attempting to create it.",
+                "%s directory: %s does not exist. Attempting to create it.",
                 fp_desc, tmp_path
             );
 
@@ -490,7 +643,7 @@ check_dir_path(const char * const filepath, const char * const fp_desc, const un
                     "Unable to create %s directory: %s (error: %i)",
                     fp_desc, tmp_path, errno
                 );
-                exit(EXIT_FAILURE);
+                return 0;
             }
 
             log_msg(LOG_ERR,
@@ -502,7 +655,7 @@ check_dir_path(const char * const filepath, const char * const fp_desc, const un
             log_msg(LOG_ERR,
                 "Stat of %s returned error %i", tmp_path, errno
             );
-            exit(EXIT_FAILURE);
+            return 0;
         }
     }
     else
@@ -514,9 +667,10 @@ check_dir_path(const char * const filepath, const char * const fp_desc, const un
             log_msg(LOG_ERR,
                 "Specified %s directory: %s is NOT a directory", fp_desc, tmp_path
             );
-            exit(EXIT_FAILURE);
+            return 0;
         }
     }
+    return 1;
 }
 
 static int
@@ -544,7 +698,7 @@ make_dir_path(const char * const run_dir)
 
             /* Stat this part of the path to see if it is a valid directory.
              * If it does not exist, attempt to create it. If it does, and
-             * it is a directory, go on.  Otherwise, any other error cause it
+             * it is a directory, go on. Otherwise, any other error cause it
              * to bail.
             */
             if(stat(tmp_path, &st) != 0)
@@ -600,11 +754,11 @@ daemonize_process(fko_srv_options_t * const opts)
     if ((pid = fork()) < 0)
     {
         perror("Unable to fork: ");
-        exit(EXIT_FAILURE);
+        clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
     }
     else if (pid != 0) /* parent */
     {
-        exit(EXIT_SUCCESS);
+        clean_exit(opts, NO_FW_CLEANUP, EXIT_SUCCESS);
     }
 
     /* Child process from here on out */
@@ -621,8 +775,7 @@ daemonize_process(fko_srv_options_t * const opts)
         fprintf(stderr,
             "[*] An instance of fwknopd is already running: (PID=%i).\n", old_pid
         );
-
-        exit(EXIT_FAILURE);
+        clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
     }
     else if(old_pid < 0)
     {
@@ -634,7 +787,7 @@ daemonize_process(fko_srv_options_t * const opts)
     */
     if ((chdir("/")) < 0) {
         perror("Could not chdir() to /: ");
-        exit(EXIT_FAILURE);
+        clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
     }
 
     /* Close un-needed file handles
@@ -676,7 +829,7 @@ write_pid_file(fko_srv_options_t *opts)
         return -1;
     }
 
-    /* Attempt to lock the PID file.  If we get an EWOULDBLOCK
+    /* Attempt to lock the PID file. If we get an EWOULDBLOCK
      * error, another instance already has the lock. So we grab
      * the pid from the existing lock file, complain and bail.
     */
@@ -770,10 +923,35 @@ get_running_pid(const fko_srv_options_t *opts)
     return(rpid);
 }
 
+#if HAVE_LIBFIU
+static void
+enable_fault_injections(fko_srv_options_t * const opts)
+{
+    if(opts->config[CONF_FAULT_INJECTION_TAG] != NULL)
+    {
+        fiu_init(0);
+        if (fiu_enable(opts->config[CONF_FAULT_INJECTION_TAG], 1, NULL, 0) != 0)
+        {
+            fprintf(stderr, "[*] Could not enable fault injection: %s\n",
+                    opts->config[CONF_FAULT_INJECTION_TAG]);
+            clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+        }
+    }
+    return;
+}
+#endif
+
 void
 clean_exit(fko_srv_options_t *opts, unsigned int fw_cleanup_flag, unsigned int exit_status)
 {
-    if(fw_cleanup_flag == FW_CLEANUP)
+#if HAVE_LIBFIU
+    if(opts->config[CONF_FAULT_INJECTION_TAG] != NULL)
+    {
+        fiu_disable(opts->config[CONF_FAULT_INJECTION_TAG]);
+    }
+#endif
+
+    if(!opts->test && (fw_cleanup_flag == FW_CLEANUP))
         fw_cleanup(opts);
 
 #if USE_FILE_CACHE
