@@ -41,7 +41,7 @@
   #include <sys/stat.h>
 #endif
 
-#if HAVE_LIBPCAP
+#if USE_LIBPCAP
   #include <pcap.h>
 #endif
 
@@ -101,6 +101,13 @@
 #define DEF_ENABLE_SPA_OVER_HTTP        "N"
 #define DEF_ENABLE_TCP_SERVER           "N"
 #define DEF_TCPSERV_PORT                "62201"
+#if USE_LIBPCAP
+  #define DEF_ENABLE_UDP_SERVER           "N"
+#else
+  #define DEF_ENABLE_UDP_SERVER           "Y"
+#endif
+#define DEF_UDPSERV_PORT                "62201"
+#define DEF_UDPSERV_SELECT_TIMEOUT      "500000" /* half a second (in microseconds) */
 #define DEF_SYSLOG_IDENTITY             MY_NAME
 #define DEF_SYSLOG_FACILITY             "LOG_DAEMON"
 
@@ -112,12 +119,34 @@
 #define RCHK_MAX_SPA_PACKET_AGE         100000  /* seconds, can disable */
 #define RCHK_MAX_SNIFF_BYTES            (2 << 14)
 #define RCHK_MAX_TCPSERV_PORT           ((2 << 16) - 1)
+#define RCHK_MAX_UDPSERV_PORT           ((2 << 16) - 1)
+#define RCHK_MAX_UDPSERV_SELECT_TIMEOUT (2 << 22)
 #define RCHK_MAX_PCAP_DISPATCH_COUNT    (2 << 22)
 #define RCHK_MAX_FW_TIMEOUT             (2 << 22)
 
+/* FirewallD-specific defines
+*/
+#if FIREWALL_FIREWALLD
+
+  #define DEF_FLUSH_FIREWD_AT_INIT         "Y"
+  #define DEF_FLUSH_FIREWD_AT_EXIT         "Y"
+  #define DEF_ENABLE_FIREWD_FORWARDING     "N"
+  #define DEF_ENABLE_FIREWD_LOCAL_NAT      "Y"
+  #define DEF_ENABLE_FIREWD_SNAT           "N"
+  #define DEF_ENABLE_FIREWD_OUTPUT         "N"
+  #define DEF_ENABLE_FIREWD_COMMENT_CHECK  "Y"
+  #define DEF_FIREWD_INPUT_ACCESS          "ACCEPT, filter, INPUT, 1, FWKNOP_INPUT, 1"
+  #define DEF_FIREWD_OUTPUT_ACCESS         "ACCEPT, filter, OUTPUT, 1, FWKNOP_OUTPUT, 1"
+  #define DEF_FIREWD_FORWARD_ACCESS        "ACCEPT, filter, FORWARD, 1, FWKNOP_FORWARD, 1"
+  #define DEF_FIREWD_DNAT_ACCESS           "DNAT, nat, PREROUTING, 1, FWKNOP_PREROUTING, 1"
+  #define DEF_FIREWD_SNAT_ACCESS           "SNAT, nat, POSTROUTING, 1, FWKNOP_POSTROUTING, 1"
+  #define DEF_FIREWD_MASQUERADE_ACCESS     "MASQUERADE, nat, POSTROUTING, 1, FWKNOP_POSTROUTING, 1"
+
+  #define RCHK_MAX_FIREWD_RULE_NUM         (2 << 15)
+
 /* Iptables-specific defines
 */
-#if FIREWALL_IPTABLES
+#elif FIREWALL_IPTABLES
 
   #define DEF_FLUSH_IPT_AT_INIT         "Y"
   #define DEF_FLUSH_IPT_AT_EXIT         "Y"
@@ -205,6 +234,9 @@ enum {
     CONF_ENABLE_SPA_OVER_HTTP,
     CONF_ENABLE_TCP_SERVER,
     CONF_TCPSERV_PORT,
+    CONF_ENABLE_UDP_SERVER,
+    CONF_UDPSERV_PORT,
+    CONF_UDPSERV_SELECT_TIMEOUT,
     CONF_LOCALE,
     CONF_SYSLOG_IDENTITY,
     CONF_SYSLOG_FACILITY,
@@ -215,7 +247,22 @@ enum {
     //CONF_EXTERNAL_CMD_ALARM,
     //CONF_ENABLE_EXT_CMD_PREFIX,
     //CONF_EXT_CMD_PREFIX,
-#if FIREWALL_IPTABLES
+#if FIREWALL_FIREWALLD
+    CONF_ENABLE_FIREWD_FORWARDING,
+    CONF_ENABLE_FIREWD_LOCAL_NAT,
+    CONF_ENABLE_FIREWD_SNAT,
+    CONF_SNAT_TRANSLATE_IP,
+    CONF_ENABLE_FIREWD_OUTPUT,
+    CONF_FLUSH_FIREWD_AT_INIT,
+    CONF_FLUSH_FIREWD_AT_EXIT,
+    CONF_FIREWD_INPUT_ACCESS,
+    CONF_FIREWD_OUTPUT_ACCESS,
+    CONF_FIREWD_FORWARD_ACCESS,
+    CONF_FIREWD_DNAT_ACCESS,
+    CONF_FIREWD_SNAT_ACCESS,
+    CONF_FIREWD_MASQUERADE_ACCESS,
+    CONF_ENABLE_FIREWD_COMMENT_CHECK,
+#elif FIREWALL_IPTABLES
     CONF_ENABLE_IPT_FORWARDING,
     CONF_ENABLE_IPT_LOCAL_NAT,
     CONF_ENABLE_IPT_SNAT,
@@ -258,6 +305,7 @@ enum {
     CONF_GPG_EXE,
     CONF_FIREWALL_EXE,
     CONF_VERBOSE,
+    CONF_FAULT_INJECTION_TAG,
 
     NUMBER_OF_CONFIG_ENTRIES  /* Marks the end and number of entries */
 };
@@ -312,7 +360,9 @@ typedef struct acc_stanza
     int                  fw_access_timeout;
     unsigned char        enable_cmd_exec;
     char                *cmd_exec_user;
+    char                *cmd_exec_group;
     uid_t                cmd_exec_uid;
+    gid_t                cmd_exec_gid;
     char                *require_username;
     unsigned char        require_source_address;
     char                *gpg_home_dir;
@@ -326,6 +376,8 @@ typedef struct acc_stanza
     unsigned char        gpg_allow_no_pw;
     char                *gpg_remote_id;
     acc_string_list_t   *gpg_remote_id_list;
+    char                *gpg_remote_fpr;
+    acc_string_list_t   *gpg_remote_fpr_list;
     time_t               access_expire_time;
     int                  expired;
     int                  encryption_mode;
@@ -349,7 +401,54 @@ typedef struct acc_stanza
 
 /* Firewall-related data and types. */
 
-#if FIREWALL_IPTABLES
+#if FIREWALL_FIREWALLD
+  /* --DSS XXX: These are arbitrary. We should determine appropriate values.
+  */
+  #define MAX_TABLE_NAME_LEN      64
+  #define MAX_CHAIN_NAME_LEN      64
+  #define MAX_TARGET_NAME_LEN     64
+
+  /* Fwknop custom chain types
+  */
+  enum {
+      FIREWD_INPUT_ACCESS,
+      FIREWD_OUTPUT_ACCESS,
+      FIREWD_FORWARD_ACCESS,
+      FIREWD_DNAT_ACCESS,
+      FIREWD_SNAT_ACCESS,
+      FIREWD_MASQUERADE_ACCESS,
+      NUM_FWKNOP_ACCESS_TYPES  /* Leave this entry last */
+  };
+
+  /* Structure to define an fwknop firewall chain configuration.
+  */
+  struct fw_chain {
+      int     type;
+      char    target[MAX_TARGET_NAME_LEN];
+      //int     direction;
+      char    table[MAX_TABLE_NAME_LEN];
+      char    from_chain[MAX_CHAIN_NAME_LEN];
+      int     jump_rule_pos;
+      char    to_chain[MAX_CHAIN_NAME_LEN];
+      int     rule_pos;
+      int     active_rules;
+      time_t  next_expire;
+  };
+
+  /* Based on the fw_chain fields (not counting type)
+  */
+  #define FW_NUM_CHAIN_FIELDS 6
+
+  struct fw_config {
+      struct fw_chain chain[NUM_FWKNOP_ACCESS_TYPES];
+      char            fw_command[MAX_PATH_LEN];
+
+      /* Flag for firewalld SNAT vs. MASQUERADE usage
+      */
+      unsigned char   use_masquerade;
+  };
+
+#elif FIREWALL_IPTABLES
   /* --DSS XXX: These are arbitrary. We should determine appropriate values.
   */
   #define MAX_TABLE_NAME_LEN      64
@@ -478,7 +577,13 @@ typedef struct fko_srv_options
     unsigned char   fw_list_all;        /* List all current firewall rules */
     unsigned char   fw_flush;           /* Flush current firewall rules */
     unsigned char   test;               /* Test mode flag */
+    unsigned char   afl_fuzzing;        /* SPA pkts from stdin for AFL fuzzing */
     unsigned char   verbose;            /* Verbose mode flag */
+    unsigned char   exit_after_parse_config; /* Parse config and exit */
+    unsigned char   enable_udp_server;  /* Enable UDP server mode */
+
+    unsigned char   firewd_disable_check_support; /* Don't use firewall-cmd ... -C */
+    unsigned char   ipt_disable_check_support; /* Don't use iptables -C */
 
     /* Flag for permitting SPA packets regardless of directionality test
      * w.r.t. the sniffing interface.  This can sometimes be useful for SPA
